#!/bin/bash
#set -x 
TOPDIR=`pwd`

KERNEL_VERSION='2.4.22'
EXTRA_VERSION='-NTGR-1.1.0.27-SP'
BUILD_DIR=${TOPDIR}/kernel_build
KERNEL_SRC_PATH=linux-${KERNEL_VERSION}
KERNEL_SRC_TARBALL=${TOPDIR}/linux-${KERNEL_VERSION}.tar.bz2
PUB_PATH='pub'
WORK_PATH='work'

# check for old kernel tarball
[ -e ${TOPDIR}/kernel-${KERNEL_VERSION}${EXTRA_VERSION} ] &&\
{
    echo "Remove old kernel dir"
    rm -rf ${TOPDIR}/kernel-${KERNEL_VERSION}${EXTRA_VERSION}
}

[ -e ${TOPDIR}/kernel-${KERNEL_VERSION}${EXTRA_VERSION}.tar.gz ] &&\
{
    echo "Remove old kernel tarball"
    rm -f ${TOPDIR}/kernel-${KERNEL_VERSION}${EXTRA_VERSION}.tar.gz
}

# check for old build dir
[ -e ${BUILD_DIR} ] && \
{
    echo "Remove old kernel_build dir"
    rm -rf ${BUILD_DIR}
}

mkdir -p ${BUILD_DIR}

# check tarball
[ ! -e $KERNEL_SRC_TARBALL ] && \
{
    echo "Can't find kernel source tarball linux-2.4.22.tar.bz2"
    exit 1
}

cd $BUILD_DIR
cp $KERNEL_SRC_TARBALL .
echo "Tar linux kernel source ..."
tar jxf $KERNEL_SRC_TARBALL
cd $KERNEL_SRC_PATH
make mrproper

cd ${TOPDIR}/${PUB_PATH}
./patch-kernel.sh ${BUILD_DIR}/${KERNEL_SRC_PATH}

cd ${BUILD_DIR}/${KERNEL_SRC_PATH}
sed -e "s@EXTRAVERSION =@EXTRAVERSION = ${EXTRA_VERSION}@g" Makefile > tempMakefile
mv tempMakefile Makefile
# We only make SP kernel
cp ${TOPDIR}/${PUB_PATH}/config-2.4.22-SP .config
make oldconfig

if [ $? -ne 0 ]; then
    echo "Make oldconfig failed!"
    exit 1
fi

# make dep, bzImage, modules
echo "Make kernel ..."
cd ${BUILD_DIR}/${KERNEL_SRC_PATH}
make dep && make bzImage && make modules
if [ $? -ne 0 ]; then
    echo "Make kernel failed!"
    exit 1
fi

# make_kernel_package
echo "Build kernel package ..."
cd ${BUILD_DIR}
mkdir -p kernel-${KERNEL_VERSION}${EXTRA_VERSION}
mkdir -p kernel-${KERNEL_VERSION}${EXTRA_VERSION}/boot

#
# Copy the kernel, system map and config file
#
cp ${KERNEL_SRC_PATH}/System.map kernel-${KERNEL_VERSION}${EXTRA_VERSION}/boot/System.map-${KERNEL_VERSION}${EXTRA_VERSION}
cp ${KERNEL_SRC_PATH}/.config kernel-${KERNEL_VERSION}${EXTRA_VERSION}/boot/config-${KERNEL_VERSION}${EXTRA_VERSION}
cp ${KERNEL_SRC_PATH}/arch/i386/boot/bzImage kernel-${KERNEL_VERSION}${EXTRA_VERSION}/boot/vmlinuz-${KERNEL_VERSION}${EXTRA_VERSION}
cd kernel-${KERNEL_VERSION}${EXTRA_VERSION}/boot
ln -sf vmlinuz-${KERNEL_VERSION}${EXTRA_VERSION} vmlinuz

# Install the kernel modules
cd ${BUILD_DIR}/${KERNEL_SRC_PATH}
make modules_install INSTALL_MOD_PATH=${BUILD_DIR}/kernel-${KERNEL_VERSION}${EXTRA_VERSION}

# Copy the other kernel modules
cd ${BUILD_DIR}
mkdir -p kernel-${KERNEL_VERSION}${EXTRA_VERSION}/lib/modules/${KERNEL_VERSION}${EXTRA_VERSION}/kernel/work
cp ${TOPDIR}/${WORK_PATH}/*.o kernel-${KERNEL_VERSION}${EXTRA_VERSION}/lib/modules/${KERNEL_VERSION}${EXTRA_VERSION}/kernel/work

#
# Depmod to create modules.dep again
#
rm -rf kernel-${KERNEL_VERSION}${EXTRA_VERSION}/lib/modules/${KERNEL_VERSION}${EXTRA_VERSION}/modules.dep
depmod -b ${BUILD_DIR}/kernel-${KERNEL_VERSION}${EXTRA_VERSION} -F ${BUILD_DIR}/kernel-${KERNEL_VERSION}${EXTRA_VERSION}/boot/System.map-${KERNEL_VERSION}${EXTRA_VERSION} -ea ${KERNEL_VERSION}${EXTRA_VERSION}

# Make the tar ball
cd ${BUILD_DIR}/kernel-${KERNEL_VERSION}${EXTRA_VERSION} && tar zcvf ../kernel-${KERNEL_VERSION}${EXTRA_VERSION}.tar.gz *

# Copy the kernel dir and kernel tarball
cd ${BUILD_DIR}
cp -r kernel-${KERNEL_VERSION}${EXTRA_VERSION} ${TOPDIR}
cp kernel-${KERNEL_VERSION}${EXTRA_VERSION}.tar.gz ${TOPDIR}
